// Functions for FTDI
#include "StdAfx.h"
#include "FTD2XX.h"
#include "daten.h"
#include "USB.h"
#include "HEX_Func.h"
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>


void ParseError(CString &str,FT_STATUS inStatus)
{
	switch ((int)inStatus)
	{
		case 0: str.Format("Command successful");
			break;
		case 1: str.Format("Invalid Handle");
			break;

		case 2: str.Format("Device not found");
			break;
				
		case 3: str.Format("Device not opened");
			break;

		case 4: str.Format("IO Error");
			break;

		case 5: str.Format("Insufficient Resources");
			break;
	
		case 6: str.Format("Invalid Parameter");
			break;

		case 7: str.Format("Invalid Baudrate");
			break;
				
		case 8: str.Format("Invalid Baudrate");
			break;
				
		case 9: str.Format("Device not opened for Erase");
			break;

		case 10: str.Format("Device not opened for Write");
			break;

		case 11: str.Format("Failed write to Device");
			break;

		case 12: str.Format("Failed write to Device");
			break;
			
		case 13: str.Format("EEPROM Read failed");
			break;

		case 14: str.Format("EEPROM Write failed");
			break;

		case 15: str.Format("EEPROM Erase failed");
			break;
						
		case 16: str.Format("EEPROM not present");
			break;

		case 17: str.Format("EEPROM not programmed");
			break;

		case 18: str.Format("Invalid Arguments");
			break;

		case 19: str.Format("Not supported");
			break;
				
		default: str.Format("Error Number %d",(int)inStatus);

	}
}


BOOL USB_Close()
{
	FT_STATUS ftStatus;
	CString str;

	if(AppData.m_USBOpen)
	{
		ftStatus = FT_Close (m_ftHandle);
		if (!ftStatus == FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant close USB Port";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}
	}

	return TRUE;
}

BOOL LIN_Init()
{
	FT_STATUS ftStatus;
	CString str,str1;

	unsigned short	CRC		= 0;

	BOOL	bFirstFrame = FALSE;

	BOOL	bSecError = FALSE;

	BOOL	bSendFrame = FALSE;

	DWORD	dwWritten;
	
//	DWORD	dwLenRxQueue;
//	DWORD	dwBytesReceived;
//	BYTE	RxBuffer[256];
	BYTE	ReadStatus[9];

	BYTE	iChar = 0;
	BYTE	iCRC = 0;
	BYTE	iCRCs = 0;

	BYTE	iIdent = 0;

	BYTE	Secure[11]	= {0x06,0x3C,0x7F,0x06,0xB1,0x3A,0x00,0x00,0x00,0xF0,0x5E};
	BYTE	Addres[11]	= {0x06,0x3C,0x7F,0x06,0xB1,0x3A,0x00,0x01,0x00,0xB1,0x5C};
	BYTE	Datas[11]	= {0x06,0x3C,0x7F,0x06,0xB1,0x3A,0x00,0x02,0x00,0x32,0x5A};
	BYTE	Status[11]	= {0x06,0x3C,0x7F,0x06,0xB1,0x3A,0x00,0x03,0x00,0x73,0x58};
	BYTE	GoDL[11]	= {0x06,0xF0,0x4C,0xFF,0x42,0xFF,0xFF,0xFF,0xFF,0xFF,0xB1};
	BYTE	SRead[2]	= {0x06,0x73};
	BYTE	SFrame[9]	= {0};


	// sending new PID, because no default value
	if(AppData.m_SR_PID != 0x73)
	{
		Status[9] = AppData.m_SR_PID;
		Status[10] = CRC_Classic(Status);
		
		//clear the ports of any data
		ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant clear USB Buffers (SR PID)";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

		ftStatus = FT_Write(m_ftHandle, Status, 11, &dwWritten);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant send Status Read PID";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

	}

	// sending new PID, because no default value
	if(AppData.m_DW_PID != 0x32)
	{
		Datas[9] = AppData.m_DW_PID;
		Datas[10] = CRC_Classic(Datas);
		
		//clear the ports of any data
		ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant clear USB Buffers (SR PID)";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

		ftStatus = FT_Write(m_ftHandle, Datas, 11, &dwWritten);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant send Data Write PID";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}
	}
	

	// sending new PID, because no default value
	if(AppData.m_AW_PID != 0xB1)
	{
		Addres[9] = AppData.m_AW_PID;
		Addres[10] = CRC_Classic(Addres);

		//clear the ports of any data
		ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant clear USB Buffers (SR PID)";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

		ftStatus = FT_Write(m_ftHandle, Addres, 11, &dwWritten);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant send Address Write PID";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}
		
	}
	
	
	// sending PID for SW 
	if(AppData.m_SW_PID != 0xF0)
		Secure[9] = AppData.m_SW_PID;
	Secure[10] = CRC_Classic(Secure);
	
	bSendFrame = USB_Send_LIN(11, Secure);
	if(!bSendFrame)
	{
		AppData.str_Message = "Cant send Secure Write PID";
		return FALSE;
	}


	// send L-Command
	GoDL[1] = AppData.m_SW_PID;
	GoDL[10] = CRC_Enhanced(GoDL);
	
	bSendFrame = USB_Send_LIN(11, GoDL);
	if(!bSendFrame)
	{
		AppData.str_Message = "Cant send Download Cmd";
		return FALSE;
	}


	SRead[1] = AppData.m_SR_PID;

	USB_Read_Status(ReadStatus);

//	str.Format("Read Status: ");

	CRC = AppData.m_SR_PID;						// frame send no PID, but PID in CS

	for(int i=0;i<8;i++)
	{
		CRC += ReadStatus[i];
		if(CRC > 255)
			CRC -= 255;
		str1.Format("%.2X ",ReadStatus[i]);
		str += str1;
	}

	iCRC = (BYTE)~CRC;							// should be value of CRC-Field

	iCRCs = ((BYTE)CRC) + ReadStatus[8];		// should be 0xFF
	
	str1.Format(" CRC:%.2X ",ReadStatus[8]);
	str += str1;

//	AfxMessageBox(str);
	
//	m_ctlLProgress.AddString(str);

	if((iCRC != ReadStatus[8]) || (iCRCs != 0xFF)) 
	{
		str.Format("Error: CRC get: %.2X  need: %.2X  calculated: %.2X",ReadStatus[8], iCRC,iCRCs);
		//m_ctlLProgress.AddString(str);
		AfxMessageBox(str);
		AppData.str_Message = " CRC Error";
		return FALSE;

	}

	AppData.m_strUSBSpeed = "19,2 KBaud";

	switch(ReadStatus[1])
	{
		case 0x32:
			AppData.str_Message = " 7032";
			//AppData.m_LIN_CPU = ReadStatus[1];
			AppData.m_LIN_CPU = 1;
			break;

		case 0x30:
			AppData.str_Message = " 7030";
			//AppData.m_LIN_CPU = ReadStatus[1];
			AppData.m_LIN_CPU = 0;
			break;

		default:
			AppData.str_Message = " unknown";
			break;
	}


	return TRUE;
}

BYTE CRC_Classic(BYTE *pAFrame)
{
	unsigned short CRC = 0;

	for(int i=2;i<10;i++)
	{
		CRC += pAFrame[i];
		if(CRC > 255)
			CRC -= 255;

	}
	return ~((BYTE)CRC);
}

BYTE CRC_Enhanced(BYTE *pAFrame)
{
	unsigned short CRC = 0;

	for(int i=1;i<10;i++)
	{
		CRC += pAFrame[i];
		if(CRC > 255)
			CRC -= 255;

	}
	return ~((BYTE)CRC);
}

BYTE CRC_En_Var(unsigned char *pData,int iPointer)
{
	unsigned short CRC = 0;

	int iPoint;

	iPoint = iPointer-9;


	for(int i=0;i<9;i++)
	{
		CRC += pData[iPoint];
		iPoint++;
		if(CRC > 255)
			CRC -= 255;

	}
	return ~((unsigned char)CRC);
}

BOOL I2C_Init() 
{
	FT_STATUS ftStatus;
	CString str,dummy;
	CString strID = "abcdefghijklmnoprstuvwxy";

	BOOL	bFirstFrame = FALSE;

	DWORD	dwWritten;
	DWORD	dwNow, dwEnd;
	
	DWORD	dwLenRxQueue;
	DWORD	dwBytesReceived;
	BYTE	RxBuffer[400];

	unsigned long	iChar = 0;

	unsigned long	iIdent = 0;

	BYTE	TxData[1] = {0x08};

 

	//clear the ports of any data
	ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant clear USB Buffers";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}


	ftStatus = FT_Write(m_ftHandle, TxData, 1, &dwWritten);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant send data";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}
	else
	{
		str = "";

		do
		{
		
			dwEnd = GetTickCount();
			dwEnd += 50;
			dwNow	= 0;

			while (dwEnd > dwNow)
				dwNow = GetTickCount();
			
			
			do
			{
				ftStatus =  FT_GetQueueStatus(m_ftHandle, &dwLenRxQueue);
				if (ftStatus != FT_OK)
				{
					ParseError(str,ftStatus);
					AppData.str_Message = "Cant get Device Status";
					AppData.strUSB_ParserError = str;
					return FALSE;
				}

			}
			while (dwLenRxQueue == 0); 

			//str.Format("USB Bytes: %d",dwLenRxQueue);
			//AfxMessageBox(str);

			ftStatus = FT_Read(m_ftHandle,RxBuffer,dwLenRxQueue,&dwBytesReceived); 
			if (ftStatus != FT_OK)
			{
				ParseError(str,ftStatus);
				AppData.str_Message = "Cant read data";
				AppData.strUSB_ParserError = str;
				return FALSE;
			}
			else
			{ 
				if(dwBytesReceived == dwLenRxQueue)
				{
					iIdent += dwLenRxQueue;

					//str.Format("USB Bytes: %d",iIdent);
					//AfxMessageBox(str);

					if(!bFirstFrame)
					{
						if((RxBuffer[0] == 0x08) || (RxBuffer[0] == 0x07))	// Timeout or NAK from Dongle
						{
							if(RxBuffer[0] == 0x08)
							{
								AppData.str_Message = "Timeout";
								AppData.strUSB_ParserError = "No response";
								return FALSE;
							}
							else
							{
								AppData.str_Message = "No Acknowlegde";
								AppData.strUSB_ParserError = "No response";
								return FALSE;
							}
						}
						else
							bFirstFrame = TRUE;
					}


					for(;iChar<dwBytesReceived;iChar++)
                    {
						if(iChar<24)   
							strID.SetAt(iChar,(TCHAR)RxBuffer[iChar]);
                    }
				}
				else
				{
					AppData.str_Message = "Length error";
					AppData.strUSB_ParserError = "Less bytes received than expected";
					return FALSE;
				}
			}
		}
		while(iIdent < 24);

		AppData.iMemSize = strID.GetAt(13)-0x30;

		if(AppData.iMemSize != 2)
		{
			AppData.iMemSize = 2;
			AfxMessageBox("MemSize corrected");
		}

		strID.SetAt(22,0x20);
		strID.SetAt(23,0x20);		
		AppData.str_Message = strID;
		AppData.m_strUSBSpeed = "100 KBaud";
		return TRUE;
	}

							
}

BOOL OnUsbCheck() 
{
	FT_STATUS ftStatus;
	CString str,dummy;
	DWORD dwWritten;

	MSG msg;

	
	DWORD dwLenRxQueue;
	DWORD dwBytesReceived;
	BYTE  RxBuffer[400];


	BYTE	TxData[1] = {0x02};

	AppData.str_Message = "";
	AppData.strUSB_ParserError = "";

	

	if(!AppData.m_USBOpen)
	{	
		ftStatus = FT_OpenEx("02042005",FT_OPEN_BY_SERIAL_NUMBER,&m_ftHandle);
		if (ftStatus == FT_OK) 
		{
			
			ftStatus = FT_SetLatencyTimer(m_ftHandle, 2);
			if (ftStatus != FT_OK)
			{
				ParseError(str,ftStatus);
				AppData.str_Message = "Cant set Latency Timer";
				AppData.strUSB_ParserError = str;
				return FALSE;
			}


			ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
			if (ftStatus != FT_OK)
			{
				ParseError(str,ftStatus);
				AppData.str_Message = "Cant clear USB Buffers";
				AppData.strUSB_ParserError = str;
				return FALSE;
			}

			AppData.m_USBOpen = TRUE;
		}
		else
		{	
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant find USB-Dongle, please check";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}
	}

	//AfxMessageBox("Port Open");

	
	if(AppData.m_USBOpen)
	{
		//clear the ports of any data
		ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant clear USB Buffers";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

	
		ftStatus = FT_Write(m_ftHandle, TxData, 1, &dwWritten);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant send Ini Data";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

		
		do
		{
			ftStatus =  FT_GetQueueStatus(m_ftHandle, &dwLenRxQueue);
			if (ftStatus != FT_OK)
			{
				ParseError(str,ftStatus);
				AppData.str_Message = "Cant get Device Status";
				AppData.strUSB_ParserError = str;
				return FALSE;
			}

			if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
		while (dwLenRxQueue == 0);

		//AfxMessageBox("Get Data");


		ftStatus = FT_Read(m_ftHandle,RxBuffer,1,&dwBytesReceived);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant read data";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}
		else
		{	
			if(AppData.m_CPU == 0)
			{
				switch(RxBuffer[0])
				{
					case 0x06:	AppData.str_Message = "Dongle Ok";
								AppData.m_Online = TRUE;
								return TRUE;
								break;

					case 0x07:	AppData.str_Message = "Dongle not Ok";
								AppData.strUSB_ParserError = "No answer from Dongle";
								return FALSE;
								break;
					
					case 0x04:	AppData.str_Message = "Detect LIN-Dongle";
								AppData.strUSB_ParserError = "Change Dongle";
								return FALSE;
								break;

					default:	AppData.str_Message = "Data Frame Error";
								AppData.strUSB_ParserError = "Timeout";
								return FALSE;
								break;
				}
			}
			else
			{
				switch(RxBuffer[0])
				{
					case 0x04:	AppData.str_Message = "Dongle Ok";
								AppData.m_Online = TRUE;
								return TRUE;
								break;

					case 0x07:	AppData.str_Message = "Dongle not Ok";
								AppData.strUSB_ParserError = "No answer from Dongle";
								return FALSE;
								break;
					
					case 0x06:	AppData.str_Message = "Detect I2C-Dongle";
								AppData.strUSB_ParserError = "Change Dongle";
								return FALSE;
								break;

					default:	AppData.str_Message = "Data Frame Error";
								AppData.strUSB_ParserError = "Timeout";
								return FALSE;
								break;
				}
			}

		}
	}
	else 
	{
		AppData.str_Message = "Dongle-Controller not Ok";
		AppData.strUSB_ParserError = "No answer from Dongle";
		return FALSE;
	}


}

BOOL USB_Setup()
{
	if(OnUsbCheck())
	{

		if(AppData.m_CPU == 0)
		{
			//AfxMessageBox("I2C");
			if(I2C_Init())	
				return TRUE;
			else
				return FALSE;
		}
		else
		{
			//AfxMessageBox("LIN");
			if(LIN_Init())
			{
				//AfxMessageBox("LIN TRUE");
				return TRUE;
			}
			else
			{	
				//AfxMessageBox("LIN FALSE");
				return FALSE;
			}

		}
	}
	else
	{
		//AfxMessageBox("OnUsbCheck False");
		return FALSE;
	}
}

BOOL USB_Run_CMD()
{
	FT_STATUS ftStatus;
	CString str,dummy;
	DWORD dwWritten;

	MSG msg;

	
	DWORD dwLenRxQueue;
	DWORD dwBytesReceived;
	BYTE  RxBuffer[2];


	BYTE	TxDataI2C[9] = {0x07,0x0E,0x05,0x52,0x00,0x08,0x00,0x00,0xA1};
	BYTE	RUN_CMD[11]	= {0x06,0xF0,0x52,0xFF,0xBD,0xFF,0xFF,0xFF,0xFF,0xFF,0xB1};

	RUN_CMD[1] = AppData.m_SW_PID;
	RUN_CMD[10] = CRC_Enhanced(RUN_CMD);


	AppData.str_Message = "";
	AppData.strUSB_ParserError = "";

	//clear the ports of any data
	ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant clear USB Buffers";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}

	if(AppData.m_CPU == 0)
		ftStatus = FT_Write(m_ftHandle, TxDataI2C, 9, &dwWritten);

	if(AppData.m_CPU == 1)
		ftStatus = FT_Write(m_ftHandle, RUN_CMD, 11, &dwWritten);

	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant send Run Command";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}

	do
	{
		ftStatus =  FT_GetQueueStatus(m_ftHandle, &dwLenRxQueue);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant get Device Status";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	while (dwLenRxQueue == 0);


	ftStatus = FT_Read(m_ftHandle,RxBuffer,1,&dwBytesReceived);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant read data";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}
	else
	{
		if(RxBuffer[0] == 0x06)
		{
			if(AppData.m_CPU == 0)
			{
				AppData.str_Message = "ARM running ...";
			}
			else
			{
				AppData.str_Message = "Kernel restart";
			}
			return TRUE;
		}
		else
		{
			if(RxBuffer[0] == 0x07)
			{
				if(AppData.m_CPU == 0)
				{
					AppData.str_Message = "RUN Command Error";
				}
				else
				{
					AppData.str_Message = "Restart Command Error";
				}

				AppData.strUSB_ParserError = "No acknowledge for Command";
				return FALSE;
			}
			else
			{
				AppData.str_Message = "Communication Error";
				AppData.strUSB_ParserError = "Timeout";
				return FALSE;
			}		
		}
	}
}

BOOL Set_USB_Speed(BYTE isLatency)
{
	FT_STATUS ftStatus;
	CString str;

//	MSG msg;


	AppData.str_Message = "";
	AppData.strUSB_ParserError = "";

	if(AppData.m_USBOpen)
	{	
			
		ftStatus = FT_SetLatencyTimer(m_ftHandle, isLatency);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant set Latency Timer";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}
		else
			return FALSE;
	}
	else
	{
		AppData.str_Message = "Cant set Latency Timer";
		AppData.strUSB_ParserError = "USB not open";
		return FALSE;
	}

}

BOOL USB_Send_Frame(DWORD dwToWrite, BYTE* TxData)
{
	FT_STATUS ftStatus;
	CString str,dummy;
	DWORD dwWritten;

	MSG msg;

	
	DWORD dwLenRxQueue;
	DWORD dwBytesReceived;
	BYTE  RxBuffer[2];

	AppData.str_Message = "";
	AppData.strUSB_ParserError = "";

	//clear the ports of any data
	ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant clear USB Buffers";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}

	
	ftStatus = FT_Write(m_ftHandle, TxData, dwToWrite, &dwWritten);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant send Data Frame";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}

	do
	{
		ftStatus =  FT_GetQueueStatus(m_ftHandle, &dwLenRxQueue);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant get Device Status";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	while (dwLenRxQueue == 0);


	ftStatus = FT_Read(m_ftHandle,RxBuffer,1,&dwBytesReceived);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant read data";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}
	else
	{
		if(RxBuffer[0] == 0x06)
		{
			return TRUE;
		}
		else
		{	
			//str.Format("Answer : %X ",RxBuffer[0]);
			//AfxMessageBox(str);
			AppData.m_Answer = RxBuffer[0];
			return FALSE;

		}

	}


}

BOOL USB_Read_Status( BYTE* RxData)
{
	FT_STATUS ftStatus;
	CString str,str1,dummy;
	DWORD dwWritten;
	DWORD dwRx = 0;
	DWORD dwRxp = 0;
	DWORD dwIndex = 0;

//	BYTE  RS[3] = {0x06,0x33,0xCC};		// default
	BYTE  RS[2] = {0x06,0x73};			// default

	RS[1] = AppData.m_SR_PID;		// actual

//	if(RS[1] != 0x33)
//		RS[2] = ~RS[1];						// new CS

//	str.Format("RS-Frame : %.2X %.2X %.2X",RS[0],RS[1],RS[2]);
//	AfxMessageBox(str);
	
	DWORD dwLenRxQueue;
	DWORD dwBytesReceived;
	BYTE  RxBuffer[10];

	AppData.str_Message = "";
	AppData.strUSB_ParserError = "";

	//clear the ports of any data
	ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant clear USB Buffers";
		AppData.strUSB_ParserError = str;
		AfxMessageBox("Cant clear USB Buffers");
		return FALSE;
	}

	
	ftStatus = FT_Write(m_ftHandle, RS, 2, &dwWritten);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant send Data Frame";
		AppData.strUSB_ParserError = str;
		AfxMessageBox("Cant send Data Frame");
		return FALSE;
	}

	do
	{
		do
		{
			ftStatus =  FT_GetQueueStatus(m_ftHandle, &dwLenRxQueue);
			if (ftStatus != FT_OK)
			{
				ParseError(str,ftStatus);
				AppData.str_Message = "Cant get Device Status";
				AppData.strUSB_ParserError = str;
				AfxMessageBox("Cant get Device Status");
				return FALSE;
			}
		}
		while (dwLenRxQueue == 0); 

		//str.Format("USB Bytes: %d",dwLenRxQueue);
		//AfxMessageBox(str);

		ftStatus = FT_Read(m_ftHandle,RxBuffer,dwLenRxQueue,&dwBytesReceived); 
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant read data";
			AppData.strUSB_ParserError = str;
			AfxMessageBox("Cant read data");
			return FALSE;
		}
		else
		{ 
			if(dwBytesReceived == dwLenRxQueue)
			{
				dwRx += dwLenRxQueue;

				//str.Format("USB Bytes: %d  ",dwRx);
				//AfxMessageBox(str);


				for(dwRxp = 0;dwRxp<dwBytesReceived;dwRxp++)
                {
					if(dwRxp<9) 
					{
							RxData[dwIndex] = RxBuffer[dwRxp];
							dwIndex++;
                    }
				}


		
			}
		}
	}
	while(dwRx < 9);


	return TRUE;
}

BOOL USB_Send_LIN(DWORD dwToWrite, BYTE* TxData)
{
	FT_STATUS ftStatus;
	CString str,dummy;
	DWORD dwWritten;

	MSG msg;

	
	DWORD dwLenRxQueue;
	DWORD dwBytesReceived;
	BYTE  RxBuffer[2];

	AppData.str_Message = "";
	AppData.strUSB_ParserError = "";

	//clear the ports of any data
	ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant clear USB Buffers";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}

	
	ftStatus = FT_Write(m_ftHandle, TxData, dwToWrite, &dwWritten);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant send Data Frame";
		AppData.strUSB_ParserError = str;
		AfxMessageBox("Sendefehler");
		return FALSE;
	}

	do
	{
		ftStatus =  FT_GetQueueStatus(m_ftHandle, &dwLenRxQueue);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant get Device Status";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	while (dwLenRxQueue == 0);


	ftStatus = FT_Read(m_ftHandle,RxBuffer,1,&dwBytesReceived);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant read data";
		AppData.strUSB_ParserError = str;
		AfxMessageBox("Empfangsfehler");
		return FALSE;
	}
	else
	{
		if(RxBuffer[0] == 0x06)
		{
			return TRUE;
		}
		else
		{	
			//str.Format("Answer : %X ",RxBuffer[0]);
			//AfxMessageBox(str);
			AppData.m_Answer = RxBuffer[0];
			return FALSE;

		}

	}


}

BOOL USB_Send_Erase(DWORD dwToWrite, BYTE* TxData)
{
	FT_STATUS ftStatus;
	CString str,dummy;
	DWORD dwWritten;

	AppData.str_Message = "";
	AppData.strUSB_ParserError = "";

	//clear the ports of any data
	ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant clear USB Buffers";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}

	
	ftStatus = FT_Write(m_ftHandle, TxData, dwToWrite, &dwWritten);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant send Data Frame";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}


	return TRUE;
}

BOOL USB_Read_Erase(void)
{
	FT_STATUS ftStatus;
	CString str,dummy;

	MSG msg;

	
	DWORD dwLenRxQueue;
	DWORD dwBytesReceived;
	BYTE  RxBuffer[2];

	AppData.str_Message = "";
	AppData.strUSB_ParserError = "";

	do
	{
		ftStatus =  FT_GetQueueStatus(m_ftHandle, &dwLenRxQueue);
		if (ftStatus != FT_OK)
		{
			ParseError(str,ftStatus);
			AppData.str_Message = "Cant get Device Status";
			AppData.strUSB_ParserError = str;
			return FALSE;
		}

		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	while (dwLenRxQueue == 0);


	ftStatus = FT_Read(m_ftHandle,RxBuffer,1,&dwBytesReceived);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant read data";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}
	else
	{
		if(RxBuffer[0] == 0x06)
		{
			return TRUE;
		}
		else
		{	
			//str.Format("Answer : %X ",RxBuffer[0]);
			//AfxMessageBox(str);
			AppData.m_Answer = RxBuffer[0];
			return FALSE;

		}

	}
}

BOOL USB_Read_DongleIdent(BYTE Ident, CString &strID)    // Stringreferenz, damit SetAt mit Punkt funktioniert
{
	FT_STATUS ftStatus;
	CString str,dummy;


	BOOL	bFirstFrame = FALSE;

	DWORD	dwWritten;
	
	DWORD	dwLenRxQueue;
	DWORD	dwBytesReceived;
	BYTE	RxBuffer[400];

	unsigned long	iChar = 0;

	unsigned long	iIdent = 0;

	BYTE	TxData[2] = {0x03};

	TxData[1] = Ident;

 

	//clear the ports of any data
	ftStatus = FT_Purge(m_ftHandle, (FT_PURGE_RX || FT_PURGE_TX));
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant clear USB Buffers";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}


	ftStatus = FT_Write(m_ftHandle, TxData, 2, &dwWritten);
	if (ftStatus != FT_OK)
	{
		ParseError(str,ftStatus);
		AppData.str_Message = "Cant send data";
		AppData.strUSB_ParserError = str;
		return FALSE;
	}
	else
	{
		str = "";

		do
		{
			
			do
			{
				ftStatus =  FT_GetQueueStatus(m_ftHandle, &dwLenRxQueue);
				if (ftStatus != FT_OK)
				{
					ParseError(str,ftStatus);
					AppData.str_Message = "Cant get Device Status";
					AppData.strUSB_ParserError = str;
					return FALSE;
				}

			}
			while (dwLenRxQueue == 0); 

			//str.Format("USB Bytes: %d",dwLenRxQueue);
			//AfxMessageBox(str);

			ftStatus = FT_Read(m_ftHandle,RxBuffer,dwLenRxQueue,&dwBytesReceived); 
			if (ftStatus != FT_OK)
			{
				ParseError(str,ftStatus);
				AppData.str_Message = "Cant read data";
				AppData.strUSB_ParserError = str;
				return FALSE;
			}
			else
			{ 
				if(dwBytesReceived == dwLenRxQueue)
				{
					iIdent += dwLenRxQueue;

					//str.Format("USB Bytes: %d",iIdent);
					//AfxMessageBox(str);


					for(;iChar<dwBytesReceived;iChar++)
                    {
						if(iChar<30)   
							strID.SetAt(iChar,(TCHAR)RxBuffer[iChar]); 
                    }
				}
				else
				{
					AppData.str_Message = "Length error";
					AppData.strUSB_ParserError = "Less bytes received than expected";
					return FALSE;
				}
			}
		}
		while(iIdent < 30);


		return TRUE;
	}
}
